package puma.control;

import puma.database.*;

/** Ermglicht die berprfung von Zugriffsrechten eines bestimmten
 *  Benutzers fr eine bestimmte Operation
 *  @author Sven Mller
 */
public class AuthentificationCheck {

   /** Ermittelt via Datenbank-Anfrage, ob fr den spezifizierten
    *  user eine Berechtigung besteht die angefragte Operation auszufhren.
    *  Falls userName null ist, liefert die Funktion immer false.
    *
    *  @param userName Der Benutzername des Benutzers.
    *  @param operation Die Bezeichnung der Operation
    *  @return true - falls die berprfung erfolgreich; false - sonst.
    *  @exception puma.database.OperationException  Falls ein Fehler bei der
                     Anfrag an die Datenbasis auftritt.
    */
   public static boolean Check (String userName, String operation) 
    throws OperationException {

      if (userName == null)
         return false;

      try {
         // Suche ID zu Benutzernamen 

         int userID = Operationen.getInstance().holePersonIDvon(userName);
         StringListeDaten zgr =
            Operationen.getInstance().holeZGR(userID);
	
         return zgr.IstVorhanden(operation);
      } catch (Exception e) {throw new OperationException("Fehler beim Check"+
                                                "der Zugriffsrechte");}
   } 
   
   /** berprft, ob das Paar UserName/Password korrekt ist
    *  @param UName Der Benutzername
    *  @param Password Das eingegebene Passwort.
    *  @return true, wenn das Password korrekt ist.
    *  @exception puma.database.OperationException Falls ein Fehler bei der
                     Anfrag an die Datenbasis auftritt.
    */
   public static boolean checkPassword(String UName, String Password) 
    throws OperationException {
      return Operationen.getInstance().checkPassword(UName, Password);
   }
}
