package puma.control;

import puma.database.*;
import java.util.Vector;

/** Command, das Weitergabe markierter Objekte umsetzt.
 *  @author Sven Mller
 */
public class WeitergabeCmd extends Command {
   /** Speichert neuen Status der weiterzugebenden Objekte
    */
   protected String newState = "";

   /** Standard-Konstruktor. Die Instanzvariable newState wird gesetzt.
    * @param newState   Der Wert fr die Instanzvariable newState.   
    * @param serverApp  Ein Rckverweis auf die aufrufende Server-Instanz.
    */  	
   public WeitergabeCmd(String newState, ServerApplication serverApp) {
      super(serverApp);  
   	this.newState = newState;
   }

  /** Fhrt die dem Command zugeordnete Aktion aus.
    *  
    * Es wird ein neues Objekt vom Typ I_StatusAendern erzeugt.Diesem werden im
    * Konstruktor ein Array von Inventarnummern (welches aus dem bergebenen
    * InType-Objekt <i>in</i> gewonnen wird), der neue Status <i>newState</i> und
    * der neue Benutzer (falls es diesen gibt - sonst null) bergeben.<br>
    * Die Methode execute() des so erzeugten Objekts liefert einen Vector mit
    * Meldungen (Strings) mit, die von <i>out</i> in das Ausgabeformat konvertiert
    * werden. Tritt ein Fehler auf wird stattdessen auf die gleiche Weise
    * eine Fehlermeldung erzeugt.
    *
    * @see I_Suchen
    * @param in Objekt, das Input-Daten liefert.
    * @param out Objekt, das fr die Darstellung der Output-Daten sorgt.
    */            
   public void execute(InType in, OutType out) {
      try {      	
      	int receiver = 0;  // Standard: Kein Empfaenger
      	try {
      	   receiver = in.getAusleiher();
      	} catch (Exception e) { receiver = 0; }
      	
         I_StatusAendern changeObj = 
            new I_StatusAendern(in.getInvNrArray(), 
               newState, receiver );
         out.setMessageArray( changeObj.execute() );
      } catch (Exception e) { out.createErrorMsg(e); }
   }   

}
