// #######################################################################
// ### generator.cc                                                    ###
// ### Testdaten-Generator                                             ###
// ### von Thomas Dreibholz, Dreibholz@bigfoot.com                     ###
// #######################################################################


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fstream.h>
#include <iostream.h>
#include <time.h>


// Zufallszahlengenerator mit Startwert von Systemzeit initialisieren
void Randomize()
{
   ifstream is("/dev/urandom",ios::in | ios::binary);
   if(is.good()) {
      unsigned int zahl;
      char   a,b,c,d;
      is >> a >> b >> c >> d;
      zahl = ((unsigned int)a << 24) + ((unsigned int)b << 16) + ((unsigned int)c << 8) + d;
      srand(zahl);
   }
   else {
      time_t zeit;
      time(&zeit);
      srand(zeit);
   }
}


const int maxCache       = 1000;
const int maxTitelPrefix =  100;
const int maxTitel       = 1000;
const int maxStaedte     =  300;
const int maxNamen       = 1000;
const int maxVornamen    =  750;

struct Liste
{
   char** Eintraege;
   int    Anzahl;
};


// Liste mit Strings aus Datei laden
Liste* LadeListe(const char* name, const int anzahl)
{
   // Liste initialisieren
   Liste* liste     = new Liste;
   liste->Eintraege = new char*[anzahl];
   liste->Anzahl    = 0;
   for(int i = 0;i < anzahl;i++)
      liste->Eintraege[i] = NULL;

   // Strings aus Datei in Liste einlesen
   ifstream is(name);
   if(is.good()) {
      char s[256];
      while(!is.eof() && (liste->Anzahl < anzahl)) {
         is.getline(s,sizeof(s));
         if(s[0] != 0x00) {
            liste->Eintraege[liste->Anzahl++] = strdup(s);
         }
      }
   }
   else {
      cerr << "Datei nicht gefunden: " << name << endl;
      exit(1);
   }
   if(liste->Anzahl < 1) {
      cerr << "Datei ist leer: " << name << endl;
      exit(1);   
   }
   return(liste); 
}


// Zuflligen String aus Liste whlen
char* Zufall(const Liste* liste) {
   const int i = random() % liste->Anzahl;
   return(liste->Eintraege[i]);
}


// Zuflligen Titel zurckgeben
char gTitel[256];
char* Titel(const Liste* prefix, const Liste* titel, int typ) {
   int i = random() % 4;
   if(i == 1)
      strcpy(gTitel,Zufall(titel));
   else
      sprintf(gTitel,"%s %s",Zufall(prefix),Zufall(titel));   
   return(gTitel);
}


// Hauptprogramm
int main(int argc, char* argv[])
{
   // Argumente auswerten
   if(argc < 4) {
      cerr << "Aufruf: " << argv[0] << " [Anzahl Namen] [Anzahl Publikationen] [Max Instanzen/Publ.]" << endl;
      return(1);
   } 
   int anz_personen = atol(argv[1]);
   int anz_publ     = atol(argv[2]);
   int max_inst     = atol(argv[3]);              
   char s[256];
   if(anz_personen < 1) anz_personen = 1;
   if(anz_publ < 1)     anz_publ = 1;
   if(max_inst < 1)     max_inst = 1;


   // Listen initialisieren
   Randomize();
   Liste* vornamen = LadeListe("Vornamen.dat",maxVornamen);
   Liste* namen    = LadeListe("Namen.dat",maxNamen);
   Liste* prefix   = LadeListe("TitelPrefix.dat",maxTitelPrefix);
   Liste* titel    = LadeListe("Titel.dat",maxTitel);
   Liste* staedte  = LadeListe("Staedte.dat",maxStaedte);
   char** buecher       = new char*[maxCache];
   char** techreps      = new char*[maxCache];
   char** zeitschriften = new char*[maxCache];
   char** instanzen     = new char*[maxCache * max_inst];
   int buch        = 0;
   int techrep     = 0;
   int zeitschrift = 0;
   int instanz     = 0;


   // Alte Tabellen entfernen   
   cerr << "Entfernen der alten Tabellen..." << endl;   
   cout << "lock tables PERSON write,INSTANZ write,BUCH write,TECHREP write,ZEITSCHRIFT write,ZUGRIFFSRECHTE write;" << endl;
   cout << "delete from PERSON;" << endl;
   cout << "delete from INSTANZ;" << endl;   
   cout << "delete from BUCH;" << endl;
   cout << "delete from TECHREP;" << endl;
   cout << "delete from ZEITSCHRIFT;" << endl;
   cout << "delete from ZUGRIFFSRECHTE;" << endl;


   // Bibliothekar hinzufgen
   cerr << anz_personen << " Personen generieren..." << endl;
   cout << "insert into PERSON values (1,'','Bibliothekar','admin','555/10001000','555/20002000','puma@puma-server','R9999');" << endl;
   cout << "insert into ZUGRIFFSRECHTE values ";
   cout << "(1,'AUSLEIH'),";
   cout << "(1,'AUSLEIHLISTE'),";
   cout << "(1,'DETAIL'),";
   cout << "(1,'RUECKGABE'),";
   cout << "(1,'WEITERGABE'),";
   cout << "(1,'SUCHEN'),";
   cout << "(1,'PUBL_LOESCHEN'),";
   cout << "(1,'PUBL_HINZUFUEGEN'),";
   cout << "(1,'PUBL_AENDERN'),";
   cout << "(1,'PERSON_LOESCHEN'),";
   cout << "(1,'PERSON_HINZUFUEGEN'),";
   cout << "(1,'PERSON_AENDERN'),";
   cout << "(1,'RECHTE_AENDERN'),";
   cout << "(1,'HISTORY_SEHEN'),";
   cout << "(1,'HISTORY_LOESCHEN');" << endl;

             
   // Personen erzeugen
   cout << "insert into PERSON values ";
   for(int i = 0;i < anz_personen;i++) {
      // Fortschritt ausgeben
      if((i % 100) == 0) {
         if((i % 5000) == 0) {
            if(i > 0) cerr << endl;
            sprintf(s,"%6d ",i);
            cerr << s;
         }
         cerr << "#";
         cerr.flush();
      }
            
      // ID, Vorname und Nachname, Passwort
      cout << "(" << 1000 + i << ",";
      if((random() % 3) != 1)
         cout << "'" << Zufall(vornamen) << " " <<  (char)(65 + (random() % 26)) << ".',";
      else
         cout << "'" << Zufall(vornamen) << "'" << ",";
      cout << "'" << Zufall(namen) << "'" << ",";
      cout << "'geheim'" << ",";     
      
      // Telefon, Fax, E-Mail und Raum
      sprintf(s,"'%03d/%08d','%03d/%08d','test%04d@puma-server','R%04d'",
                random() % 1000,random() % 100000000,random() % 1000,random() % 100000000,i,i+1);  
      cout << s << ")";
      
      if(i < (anz_personen - 1)) cout << ",";
   }    
   cout << ";" << endl;
   cerr << endl;


   // Zugriffsrechte eintragen
   cerr << "Zugriffsrechte eintragen..." << endl;
   cout << "insert into ZUGRIFFSRECHTE values ";
   for(int i = 0;i < anz_personen;i++) {
      // Fortschritt ausgeben
      if((i % 100) == 0) {
         if((i % 5000) == 0) {
            if(i > 0) cerr << endl;
            sprintf(s,"%6d ",i);
            cerr << s;
         }
         cerr << "#";
         cerr.flush();
      }

      // Zugriffsrechte      
      cout << "(" << 1000+i << ",'AUSLEIH'),";
      cout << "(" << 1000+i << ",'AUSLEIHLISTE'),";
      cout << "(" << 1000+i << ",'DETAIL'),";
      cout << "(" << 1000+i << ",'RUECKGABE'),";
      cout << "(" << 1000+i << ",'WEITERGABE'),";
      cout << "(" << 1000+i << ",'SUCHEN')";
      
      if(i < (anz_personen - 1)) cout << ",";
   }    
   cout << ";" << endl;
   cerr << endl;
   

   // Publikationen erzeugen   
   cerr << anz_publ << " Publikationen generieren..." << endl;         
   int inv = 1;
   int sig = 1;
   int cache = 0;
   for(int i = 0;i < anz_publ;i++) {
      // Fortschritt ausgeben
      if((i % 100) == 0) {
         if((i % 5000) == 0) {
            if(i > 0) cerr << endl;
            sprintf(s,"%6d ",i);
            cerr << s;
         }
         cerr << "#";
         cerr.flush();
      }
   
      int typ = random() % 3;
      switch(typ) {
         case 0:   // Buch einfgen
            sprintf(s,"'S-%06d','%s','%s, %s',%d,'deutsch','ISBN-%04d-%04d','Auflage %d','%s-Verlag'",
               sig, Titel(prefix,titel,0),
               Zufall(namen), Zufall(vornamen),
               1960+(random() % 50),random() % 10000,random() % 10000, random() % 20,
               Zufall(namen));
            buecher[buch++] = strdup(s);
          break;
         case 1:   // TechRep einfgen
            sprintf(s,"'S-%06d','%s','%s, %s',%d,'deutsch','ISSN-%04d-%04d',%d,'Universitt %s'",
               sig, Titel(prefix,titel,1),
               Zufall(namen), Zufall(vornamen),
               1960+(random() % 50),random() % 10000,random() % 10000, random() % 20,
               Zufall(staedte));
            techreps[techrep++] = strdup(s);
          break;
         case 2:   // Zeitschrift einfgen
            sprintf(s,"'S-%06d','%s',%d,%d,'deutsch','%s-Verlag'",
               sig, Titel(prefix,titel,2),
               1+(random() % 51), 1960+(random() % 50),Zufall(namen));
            zeitschriften[zeitschrift++] = strdup(s);   
          break;         
      }
      
      // Instanzen hinzufgen
      int anzahl = random() % max_inst;
      if(anzahl < 1) anzahl = 1;
      for(int j = 0;j < anzahl;j++) {
         sprintf(s,"'I-%07d','S-%06d','VERFUEGBAR',0",inv++,sig);
         instanzen[instanz++] = strdup(s);
      }   
      sig++;
      cache++;
         
      // Cache leeren, wenn Zhler > maxCache   
      if(cache > maxCache) {
         cache = 0;
         
         if(buch > 0) {  
            cout << "insert into BUCH values ";
            for(int i = 0;i < buch;i++) {
               cout << "(" << buecher[i] << ")";
               free(buecher[i]);
               if(i < (buch - 1)) cout << ",";
            }
            cout << ";" << endl;
            buch = 0;
         } 
         if(techrep > 0) {
            cout << "insert into TECHREP values ";
            for(int i = 0;i < techrep;i++) {
               cout << "(" << techreps[i] << ")";
               free(techreps[i]);
               if(i < (techrep - 1)) cout << ",";
            }
            cout << ";" << endl;
            techrep = 0;
         }
         if(zeitschrift > 0) {
            cout << "insert into ZEITSCHRIFT values ";
            for(int i = 0;i < zeitschrift;i++) {
               cout << "(" << zeitschriften[i] << ")";
               free(zeitschriften[i]);
               if(i < (zeitschrift - 1)) cout << ",";
            }
            cout << ";" << endl;
            zeitschrift = 0;
         }
         if(instanz > 0) {   
            cout << "insert into INSTANZ values ";
            for(int i = 0;i < instanz;i++) {
               cout << "(" << instanzen[i] << ")";
               free(instanzen[i]);
               if(i < (instanz - 1)) cout << ",";
            }
            cout << ";" << endl;
            instanz = 0;
         }
      }      
   }
   cerr << endl;
      
   // Cache leeren
   if(buch > 0) {  
      cout << "insert into BUCH values ";
      for(int i = 0;i < buch;i++) {
         cout << "(" << buecher[i] << ")";
         free(buecher[i]);
         if(i < (buch - 1)) cout << ",";
      }
      cout << ";" << endl;
      buch = 0;
   } 
   if(techrep > 0) {
      cout << "insert into TECHREP values ";
      for(int i = 0;i < techrep;i++) {
         cout << "(" << techreps[i] << ")";
         free(techreps[i]);
         if(i < (techrep - 1)) cout << ",";
      }
      cout << ";" << endl;
      techrep = 0;
   }
   if(zeitschrift > 0) {
      cout << "insert into ZEITSCHRIFT values ";
      for(int i = 0;i < zeitschrift;i++) {
         cout << "(" << zeitschriften[i] << ")";
         free(zeitschriften[i]);
         if(i < (zeitschrift - 1)) cout << ",";
      }
      cout << ";" << endl;
      zeitschrift = 0;
   }
   if(instanz > 0) {   
      cout << "insert into INSTANZ values ";
      for(int i = 0;i < instanz;i++) {
         cout << "(" << instanzen[i] << ")";
         free(instanzen[i]);
         if(i < (instanz - 1)) cout << ",";
      }
      cout << ";" << endl;
      instanz = 0;
   }

  
   cerr << "Optimieren der Tabellen..." << endl; 
   cout << "optimize table PERSON;" << endl;
   cout << "optimize table INSTANZ;" << endl;
   cout << "optimize table BUCH;" << endl;
   cout << "optimize table TECHREP;" << endl;
   cout << "optimize table ZEITSCHRIFT;" << endl;
   cout << "optimize table ZUGRIFFSRECHTE;" << endl;
   cout << "unlock tables;" << endl;
   cerr << "Ich habe fertig!" << endl;
   return(0);
}
